#!/usr/bin/env python3
import json, webbrowser
from .config import config
from musixmatch import Musixmatch
import click
from textwrap import fill

cfg = config()
key = cfg.get_key()
musixmatch = Musixmatch(key)

class msx:

    def artists(country):
        artist_json = musixmatch.chart_artists(country = country, page = 1, page_size = 20)
        artist_list = json.loads(json.dumps(artist_json['message']['body']['artist_list']))
        for i in range(0, len(artist_list)):
            wrapped = fill(artist_list[i]['artist']['artist_name'], width=71, subsequent_indent=' '*7)
            print('     ' + wrapped)

    def songs(country):
        songs_json = musixmatch.chart_tracks_get(country = country, page = 1, page_size = 20, f_has_lyrics = 1)
        songs_list = json.loads(json.dumps(songs_json['message']['body']['track_list']))
        for i in range(0, len(songs_list)):
            wrapped = fill(songs_list[i]['track']['track_name'], width=71, subsequent_indent=' '*7)
            print('     ' + click.style('Song: ', bold = True, fg = 'green') + wrapped)
            wrapped = fill(songs_list[i]['track']['artist_name'], width=71, subsequent_indent=' '*7)
            print('     ' + click.style('Album: ', bold = True, fg = 'blue') + songs_list[i]['track']['album_name'] + '\n' + click.style('     Artist: ', bold = True, fg = 'yellow') + wrapped + '\n')

    def search(song, artist):
        if song != '' or artist != '':
            search_val = musixmatch.track_search(q_artist = artist, q_track = song, page = 1, page_size = 10, s_track_rating='desc')
            search_list = json.loads(json.dumps(search_val['message']['body']['track_list']))
            if len(search_list) == 0:
                print('     ' + 'No Data Available')
            for i in range(0, len(search_list)):
                genre = json.loads(json.dumps(search_list[i]['track']['primary_genres']['music_genre_list']))
                wrapped = fill(search_list[i]['track']['track_name'], width=71, subsequent_indent=' '*7)
                print('     ' + click.style('Song: ', bold = True, fg = 'green') + wrapped)
                wrapped = fill(search_list[i]['track']['artist_name'], width=71, subsequent_indent=' '*7)
                print('     ' + click.style('Album: ', bold = True, fg = 'blue') + search_list[i]['track']['album_name'] + '\n' + click.style('     Artist: ', bold = True, fg = 'yellow') + wrapped)
                if len(genre) != 0:
                    wrapped = fill(genre[0]['music_genre']['music_genre_name'], width=71, subsequent_indent=' '*7)
                    print('     ' + click.style('Genre: ', bold = True, fg = 'red') + wrapped + '\n')
                else:
                    print('     ' + click.style('Genre: ', bold = True, fg = 'red') + 'No genre available\n')
        else:
            print('     ' + click.style('No song name or artist provided', blink = True, fg = 'magenta'))

    def albums(artist_name):
        if artist_name != '':
            track_val = musixmatch.track_search(q_artist = artist_name, q_track = '', page = 1, page_size = 1, s_track_rating='desc')
            track_list = json.loads(json.dumps(track_val['message']['body']['track_list']))
            artist_id = track_list[0]['track']['artist_id']
            artist_val = musixmatch.artist_albums_get(artist_id = artist_id, page = 1, page_size = 100, s_release_date = 'desc', g_album_name = 1)
            album_list = json.loads(json.dumps(artist_val['message']['body']['album_list']))
            if len(album_list) == 0:
                print('     ' + 'No Album Data Available')
            for i in range(0, len(album_list)):
                wrapped = fill(album_list[i]['album']['album_name'], width=71, subsequent_indent=' '*7)
                print('     ' + click.style('Album Name: ', bold = True, fg = 'green') + wrapped)
                print('     ' + click.style('Number of Tracks: ', bold = True, fg = 'blue') + str(album_list[i]['album']['album_track_count']) + '\n' + click.style('       Release Date: ', bold = True, fg = 'yellow') + album_list[i]['album']['album_release_date'] + '\n')
        else:
            print('     ' + click.style('No artist provided', blink = True, fg = 'magenta'))

    def related_artists(artist_name, number):
        if artist_name != '':
            track_val = musixmatch.track_search(q_artist = artist_name, q_track = '', page = 1, page_size = 1, s_track_rating='desc')
            track_list = json.loads(json.dumps(track_val['message']['body']['track_list']))
            artist_id = track_list[0]['track']['artist_id']
            r_artist_val = musixmatch.artist_related_get(artist_id = artist_id, page = 1, page_size = number)
            r_artist_list = json.loads(json.dumps(r_artist_val['message']['body']['artist_list']))
            print('Artists Similar to ' + artist_name + ' :\n')
            if len(r_artist_list) == 0:
                print('     ' + 'No Data Available')
            for i in range(0, len(r_artist_list)):
                wrapped = fill(r_artist_list[i]['artist']['artist_name'], width=71, subsequent_indent=' '*7)
                print('     ' + click.style('Artist Name: ', bold = True, fg = 'green') + wrapped + '\n')
        else:
            print('     ' + click.style('No artist provided', blink = True, fg = 'magenta'))

    def lyrics(song, artist_name, browser):
        if song != '' and artist_name != '':
            match_val = musixmatch.matcher_track_get(q_track = song, q_artist = artist_name)
            if browser == 'yes':
                match_val = musixmatch.matcher_track_get(q_track = song, q_artist = artist_name)
                match_url = json.loads(json.dumps(match_val['message']['body']['track']['track_share_url']))
                webbrowser.open(match_url)
            elif browser == 'no':
                match_val = musixmatch.matcher_lyrics_get(q_track = song, q_artist = artist_name)
                match_lyrics = json.loads(json.dumps(match_val['message']['body']['lyrics']['lyrics_body']))
                click.echo_via_pager(match_lyrics)
        else:
            print('     ' + click.style('Song name and artist not provided', blink = True, fg = 'magenta'))



#!/usr/bin/env python3
from prompt_toolkit.contrib.completers import WordCompleter
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.interface import CommandLineInterface, AcceptAction
from prompt_toolkit.application import Application
from prompt_toolkit.key_binding.manager import KeyBindingManager
from prompt_toolkit.shortcuts import create_default_layout, create_eventloop
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.filters import Always
from prompt_toolkit.keys import Keys
from pygments.token import Token
from pygments.styles.tango import TangoStyle
from prompt_toolkit.styles import style_from_pygments
import subprocess, sys

class musixli:
    musix_completer = WordCompleter(['artists', 'songs', 'search', 'albums', 'related_artists', 'lyrics', 'clear', '--country', '--song', '--artist-name', '--number', '--browser'], ignore_case = True)
    def __init__(self):
        def get_toolbar(self):
            return [(Token.Toolbar.Status.Key, '[msx] Help      [ctrl+q] Exit')]

        updated_style = style_from_pygments(TangoStyle, {
            Token.Menu.Completions.Completion.Current: 'bg:#acba36 #000000',
            Token.Menu.Completions.Completion: 'bg:#008888 #ffffff',
            Token.Menu.Completions.ProgressBar: 'bg:#acba36',
            Token.Scrollbar: 'bg:#acba36',
            Token.Scrollbar.Button: 'bg:#003333',
            Token.Toolbar: '#ffffff bg:#333333',
            Token: '#00ffff bold',
            Token.Toolbar.Status.Key: '#ff0000'
        })
        history = InMemoryHistory()
        layout = create_default_layout(
            message = u'musixli:$>> ',
            get_bottom_toolbar_tokens = get_toolbar
        )
        cli_buffer = Buffer(
            history = history,
            auto_suggest = AutoSuggestFromHistory(),
            enable_history_search=True,
            completer = self.musix_completer,
            complete_while_typing = Always(),
            accept_action=AcceptAction.RETURN_DOCUMENT
            )
        loop = create_eventloop()
        self.manager = KeyBindingManager()

        @self.manager.registry.add_binding(Keys.ControlQ, eager = True)
        def exit_(event):
            sys.exit()

        application = Application(  key_bindings_registry = self.manager.registry,
                                    layout = layout,
                                    mouse_support = False,
                                    buffer = cli_buffer,
                                    style = updated_style,
                                    ignore_case = True,
                                    )
        self.cli = CommandLineInterface(application = application, eventloop = loop)

    def commands(self,document):
        command = document.text
        subprocess.call(command, shell = True)

    def start(self):
        while True:
            document = self.cli.run(reset_current_buffer=True)
            self.commands(document)



#!/usr/bin/env python3
import click
from .msx import msx

class msx_cli:
    @click.group()
    def begin():
        '''
        A little tool that lets you search for different artists, albums, songs and their lyrics with just entering certain commands.
        '''

    @begin.command()
    @click.option('--country', default = 'US' , help = 'Specify the country, default is US.')
    def artists(country):
        '''
        Lists top 20 artists.
        '''
        msx.artists(country)

    @begin.command()
    @click.option('--country', default = 'US' , help = 'Specify the country, default is US.')
    def songs(country):
        '''
        Lists top 20 songs.
        '''
        msx.songs(country)

    @begin.command()
    @click.option('--song', default = '', help = 'Search by song name. Must be in quotes.')
    @click.option('--artist-name', default = '', help = 'Search by artist name. Must be in quotes.')
    def search(song, artist_name):
        '''
        Provides a detailed list based on the query.
        '''
        msx.search(song, artist_name)

    @begin.command()
    @click.option('--artist-name', default = '', help = 'Search by artist name. Must be in quotes.')
    def albums(artist_name):
        '''
        Provides a list of albums associated with an artist
        '''
        msx.albums(artist_name)

    @begin.command()
    @click.option('--artist-name', default = '', help = 'Search by artist name. Must be in quotes.')
    @click.option('--number', default = 5, help = 'Specify the number of artists. Default is 5.')
    def related_artists(artist_name, number):
        '''
        Provides a list of artists similar to the specified artist
        '''
        msx.related_artists(artist_name, number)

    @begin.command()
    @click.option('--song', default = '', help = 'Specify the name of the song.')
    @click.option('--artist-name', default = '', help = 'Specify the name of the artist')
    @click.option('--browser', default = 'no', help = 'Open in a browser or not: (yes/no). Default is no', type = click.Choice(['yes', 'no']))
    def lyrics(song, artist_name, browser):
        '''
        Opens a snippet of the lyrics of a specified song in the window itself or open the whole lyrics in the browser.
        '''
        msx.lyrics(song, artist_name, browser)

    @begin.command()
    def clear():
        '''
        Clears the entire screen.
        '''
        click.clear()



#!/usr/bin/env python3
from setuptools import setup, find_packages
with open("README.md", "r") as fh:
    long_description = fh.read()
setup(
    name = 'musixli',
    version = '2.1.0',
    description = 'A command line application that lets you search for different artists, albums, songs and their lyrics.',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    author = 'Yathartha Goel',
    license = 'MIT License',
    platforms = 'Cross Platform',
    packages = find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires = [
        'click',
        'pymusixmatch',
        'prompt_toolkit == 1.0.15',
        'pygments'
    ],
    entry_points = {
        "console_scripts": ['musixli = musix.main:cli',
                            'msx = musix.main_msx:cli']
        }
    )
